##Procedure Header
# Name:
#    ixiangpf::emulation_bgp_flow_spec_config
#
# Description:
#    This procedure configures BGP IPv4 and IPv6 Flow Spec.
#
# Synopsis:
#    ixiangpf::emulation_bgp_flow_spec_config
#        -mode    CHOICES create modify enable
#        [-handle ANY]
#
# Arguments:
#    -mode
#        This option defines the action to be taken on the BGP server.
#    -handle
#        bgp_flowSpecV4_handle: -handle is returned by procedure emulation_bgp_flow_spec_config when a IPv4 Flow Spec is created or modified
#        bgp_flowSpecV6_handle: -handle is returned by procedure emulation_bgp_flow_spec_config when a IPv6 Flow Spec is created or modified
#
# Return Values:
#    $::SUCCESS | $::FAILURE
#    key:status                 value:$::SUCCESS | $::FAILURE
#    When status is $::FAILURE, contains more information
#    key:log                    value:When status is $::FAILURE, contains more information
#    Handle of bgpipv4flowspec configured
#    key:bgp_flowSpecV4_handle  value:Handle of bgpipv4flowspec configured
#    Item Handle of any bgpFlowSpecRangesListV4 or bgpFlowSpecRangesListV6 configured
#    key:handles                value:Item Handle of any bgpFlowSpecRangesListV4 or bgpFlowSpecRangesListV6 configured
#
# Examples:
#    See files starting with BGP_ in the Samples subdirectory.  Also see some of the L2VPN, L3VPN, MPLS, and MVPN sample files for further examples of the BGP usage.
#    See the BGP example in Appendix A, "Example APIs," for one specific example usage.
#
# Sample Input:
#
# Sample Output:
#    {status $::SUCCESS}
#    {bgp_flowSpecV4_handle {/topology:1/deviceGroup:1/ethernet:1/ipv4:1/bgpIpv4Peer:1/bgpFlowSpecRangesListV4:1/} {/topology:1/deviceGroup:1/ethernet:1/ipv6:1/bgpIpv6Peer:1/bgpFlowSpecRangesListV4:1/}}
#    {bgp_flowSpecV6_handle {/topology:1/deviceGroup:1/ethernet:1/ipv4:1/bgpIpv4Peer:1/bgpFlowSpecRangesListV6:1/} {/topology:1/deviceGroup:1/ethernet:1/ipv6:1/bgpIpv6Peer:1/bgpFlowSpecRangesListV6:1/}}
#
# Notes:
#    Coded versus functional specification.
#    When a -handle is provided with a BGP/BGP+ protocol stack handle or a protocol session handle, the api will create IPv4/IPv6 Flow Spec
#    When a -handle is provided with a BGP Flow Spec Handle, the api will modify the created IPv4/IPv6 Flow Spec
#
# See Also:
#

package ixiangpf;

use utils;
use ixiahlt;

sub emulation_bgp_flow_spec_config {

	my $args = shift(@_);

	my @notImplementedParams = ();
	my @mandatoryParams = ();
	my @fileParams = ();

	# ixiahlt::logHltapiCommand('emulation_bgp_flow_spec_config', $args);
	# ixiahlt::utrackerLog ('emulation_bgp_flow_spec_config', $args);

	return ixiangpf::runExecuteCommand('emulation_bgp_flow_spec_config', \@notImplementedParams, \@mandatoryParams, \@fileParams, $args);
}

# Return value for the package
return 1;
